@@ -35,9 +35,11 @@ make clean || exit 1
 
 "${covbuild}" --dir cov-int make -j${cpus} || exit 1
 
+echo >&2 "Compressing data..."
 tar czvf netdata-coverity-analysis.tgz cov-int || exit 1
 
-curl --form token="${token}" \
+echo >&2 "Sending analysis..."
+curl --progress-bar --form token="${token}" \
   --form email=costa@tsaousis.gr \
   --form file=@netdata-coverity-analysis.tgz \
   --form version="${version}" \
