@@ -351,8 +351,8 @@ inline void health_alarm_log(
         time_t duration,
         calculated_number old_value,
         calculated_number new_value,
-        int old_status,
-        int new_status,
+        RRDCALC_STATUS old_status,
+        RRDCALC_STATUS new_status,
         const char *source,
         const char *units,
         const char *info,
