@@ -1,4 +1,5 @@
 #include "common.h"
+#include <Availability.h>
 #include <sys/sysctl.h>
 // NEEDED BY: do_bandwidth
 #include <net/route.h>
@@ -112,9 +113,21 @@ int do_macos_sysctl(int update_every, usec_t dt) {
     /*
      * Dirty workaround for /usr/include/netinet6/ip6_var.h absence.
      * Struct ip6stat was copied from bsd/netinet6/ip6_var.h from xnu sources.
+     * Do the same for previously missing scope6_var.h on OS X < 10.11.
      */
 #define	IP6S_SRCRULE_COUNT 16
+
+#if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED < 101100)
+#ifndef _NETINET6_SCOPE6_VAR_H_
+#define _NETINET6_SCOPE6_VAR_H_
+#include <sys/appleapiopts.h>
+
+#define	SCOPE6_ID_MAX	16
+#endif
+#else
 #include <netinet6/scope6_var.h>
+#endif
+
     struct	ip6stat {
         u_quad_t ip6s_total;		/* total packets received */
         u_quad_t ip6s_tooshort;		/* packet too short */
@@ -217,9 +230,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                 st = rrdset_find_bytype_localhost("system", "load");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("system", "load", NULL, "load", NULL, "System Load Average", "load"
-                                                 , 100, (update_every < MIN_LOADAVG_UPDATE_EVERY)
-                                                        ? MIN_LOADAVG_UPDATE_EVERY : update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "system"
+                            , "load"
+                            , NULL
+                            , "load"
+                            , NULL
+                            , "System Load Average"
+                            , "load"
+                            , "macos"
+                            , "sysctl"
+                            , 100
+                            , (update_every < MIN_LOADAVG_UPDATE_EVERY) ? MIN_LOADAVG_UPDATE_EVERY : update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrddim_add(st, "load1", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
                     rrddim_add(st, "load5", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
                     rrddim_add(st, "load15", NULL, 1, 1000, RRD_ALGORITHM_ABSOLUTE);
@@ -246,8 +270,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         } else {
             st = rrdset_find_localhost("system.swap");
             if (unlikely(!st)) {
-                st = rrdset_create_localhost("system", "swap", NULL, "swap", NULL, "System Swap", "MB", 201
-                                             , update_every, RRDSET_TYPE_STACKED);
+                st = rrdset_create_localhost(
+                        "system"
+                        , "swap"
+                        , NULL
+                        , "swap"
+                        , NULL
+                        , "System Swap"
+                        , "MB"
+                        , "macos"
+                        , "sysctl"
+                        , 201
+                        , update_every
+                        , RRDSET_TYPE_STACKED
+                );
                 rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                 rrddim_add(st, "free",    NULL, 1, 1048576, RRD_ALGORITHM_ABSOLUTE);
@@ -296,8 +332,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 }
                 st = rrdset_find_localhost("system.ipv4");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("system", "ipv4", NULL, "network", NULL, "IPv4 Bandwidth", "kilobits/s"
-                                                 , 500, update_every, RRDSET_TYPE_AREA);
+                    st = rrdset_create_localhost(
+                            "system"
+                            , "ipv4"
+                            , NULL
+                            , "network"
+                            , NULL
+                            , "IPv4 Bandwidth"
+                            , "kilobits/s"
+                            , "macos"
+                            , "sysctl"
+                            , 500
+                            , update_every
+                            , RRDSET_TYPE_AREA
+                    );
 
                     rrddim_add(st, "InOctets",  "received", 8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutOctets", "sent",    -8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
@@ -334,8 +382,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_tcp_packets)) {
                 st = rrdset_find_localhost("ipv4.tcppackets");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcppackets", NULL, "tcp", NULL, "IPv4 TCP Packets"
-                                                 , "packets/s", 2600, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcppackets"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "IPv4 TCP Packets"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2600
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InSegs", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutSegs", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -352,8 +412,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_tcp_errors)) {
                 st = rrdset_find_localhost("ipv4.tcperrors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcperrors", NULL, "tcp", NULL, "IPv4 TCP Errors", "packets/s"
-                                                 , 2700, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcperrors"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "IPv4 TCP Errors"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2700
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "InErrs", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -373,8 +445,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_tcp_handshake)) {
                 st = rrdset_find_localhost("ipv4.tcphandshake");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcphandshake", NULL, "tcp", NULL, "IPv4 TCP Handshake Issues"
-                                                 , "events/s", 2900, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcphandshake"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "IPv4 TCP Handshake Issues"
+                            , "events/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2900
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "EstabResets", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -397,8 +481,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_tcpext_connaborts = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv4.tcpconnaborts");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcpconnaborts", NULL, "tcp", NULL, "TCP Connection Aborts"
-                                                 , "connections/s", 3010, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcpconnaborts"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "TCP Connection Aborts"
+                            , "connections/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3010
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "TCPAbortOnData",    "baddata",     1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "TCPAbortOnClose",   "userclosed",  1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -420,8 +516,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_tcpext_ofo = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv4.tcpofo");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcpofo", NULL, "tcp", NULL, "TCP Out-Of-Order Queue"
-                                                 , "packets/s", 3050, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcpofo"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "TCP Out-Of-Order Queue"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3050
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "TCPOFOQueue", "inqueue",  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 }
@@ -438,8 +546,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                 st = rrdset_find_localhost("ipv4.tcpsyncookies");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "tcpsyncookies", NULL, "tcp", NULL, "TCP SYN Cookies"
-                                                 , "packets/s", 3100, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "tcpsyncookies"
+                            , NULL
+                            , "tcp"
+                            , NULL
+                            , "TCP SYN Cookies"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3100
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "SyncookiesRecv",   "received",  1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "SyncookiesSent",   "sent",     -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -455,12 +575,26 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
             // --------------------------------------------------------------------
 
+
+#if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101100)
             if (do_ecn == CONFIG_BOOLEAN_YES || (do_ecn == CONFIG_BOOLEAN_AUTO && (tcpstat.tcps_ecn_recv_ce || tcpstat.tcps_ecn_not_supported))) {
                 do_ecn = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv4.ecnpkts");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "ecnpkts", NULL, "ecn", NULL, "IPv4 ECN Statistics"
-                                                 , "packets/s", 8700, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "ecnpkts"
+                            , NULL
+                            , "ecn"
+                            , NULL
+                            , "IPv4 ECN Statistics"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 8700
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "InCEPkts", "CEP", 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -472,6 +606,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 rrddim_set(st, "InNoECTPkts", tcpstat.tcps_ecn_not_supported);
                 rrdset_done(st);
             }
+#endif
 
         }
     }
@@ -489,8 +624,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_udp_packets)) {
                 st = rrdset_find_localhost("ipv4.udppackets");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "udppackets", NULL, "udp", NULL, "IPv4 UDP Packets"
-                                                 , "packets/s", 2601, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "udppackets"
+                            , NULL
+                            , "udp"
+                            , NULL
+                            , "IPv4 UDP Packets"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2601
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InDatagrams", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutDatagrams", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -507,23 +654,41 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_udp_errors)) {
                 st = rrdset_find_localhost("ipv4.udperrors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "udperrors", NULL, "udp", NULL, "IPv4 UDP Errors", "events/s"
-                                                 , 2701, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "udperrors"
+                            , NULL
+                            , "udp"
+                            , NULL
+                            , "IPv4 UDP Errors"
+                            , "events/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2701
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "RcvbufErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "NoPorts", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InCsumErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+#if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1090)
                     rrddim_add(st, "IgnoredMulti", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
+#endif
                 } else
                     rrdset_next(st);
 
                 rrddim_set(st, "InErrors", udpstat.udps_hdrops + udpstat.udps_badlen);
                 rrddim_set(st, "NoPorts", udpstat.udps_noport);
                 rrddim_set(st, "RcvbufErrors", udpstat.udps_fullsock);
+#if (defined __MAC_OS_X_VERSION_MIN_REQUIRED && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1090)
                 rrddim_set(st, "InCsumErrors", udpstat.udps_badsum + udpstat.udps_nosum);
                 rrddim_set(st, "IgnoredMulti", udpstat.udps_filtermcast);
+#else
+                rrddim_set(st, "InCsumErrors", udpstat.udps_badsum);
+#endif
                 rrdset_done(st);
             }
         }
@@ -550,8 +715,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_icmp_packets)) {
                 st = rrdset_find_localhost("ipv4.icmp");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "icmp", NULL, "icmp", NULL, "IPv4 ICMP Packets", "packets/s"
-                                                 , 2602, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "icmp"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv4 ICMP Packets"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2602
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InMsgs", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutMsgs", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -567,8 +744,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                 st = rrdset_find_localhost("ipv4.icmp_errors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "icmp_errors", NULL, "icmp", NULL, "IPv4 ICMP Errors"
-                                                 , "packets/s", 2603, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "icmp_errors"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv4 ICMP Errors"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2603
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutErrors", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -588,8 +777,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_icmpmsg)) {
                 st = rrdset_find_localhost("ipv4.icmpmsg");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "icmpmsg", NULL, "icmp", NULL, "IPv4 ICMP Messages"
-                                                 , "packets/s", 2604, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "icmpmsg"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv4 ICMP Messages"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 2604
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InEchoReps", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutEchoReps", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -625,8 +826,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_ip_packets)) {
                 st = rrdset_find_localhost("ipv4.packets");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "packets", NULL, "packets", NULL, "IPv4 Packets", "packets/s"
-                                                 , 3000, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "packets"
+                            , NULL
+                            , "packets"
+                            , NULL
+                            , "IPv4 Packets"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3000
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InReceives", "received", 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutRequests", "sent", -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -647,8 +860,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_ip_fragsout)) {
                 st = rrdset_find_localhost("ipv4.fragsout");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "fragsout", NULL, "fragments", NULL, "IPv4 Fragments Sent"
-                                                 , "packets/s", 3010, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "fragsout"
+                            , NULL
+                            , "fragments"
+                            , NULL
+                            , "IPv4 Fragments Sent"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3010
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "FragOKs", "ok", 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -668,8 +893,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_ip_fragsin)) {
                 st = rrdset_find_localhost("ipv4.fragsin");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "fragsin", NULL, "fragments", NULL, "IPv4 Fragments Reassembly"
-                                                 , "packets/s", 3011, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "fragsin"
+                            , NULL
+                            , "fragments"
+                            , NULL
+                            , "IPv4 Fragments Reassembly"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3011
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "ReasmOKs", "ok", 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -689,8 +926,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             if (likely(do_ip_errors)) {
                 st = rrdset_find_localhost("ipv4.errors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv4", "errors", NULL, "errors", NULL, "IPv4 Errors", "packets/s"
-                                                 , 3002, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv4"
+                            , "errors"
+                            , NULL
+                            , "errors"
+                            , NULL
+                            , "IPv4 Errors"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3002
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "InDiscards", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -734,8 +983,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_ip6_packets = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.packets");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "packets", NULL, "packets", NULL, "IPv6 Packets", "packets/s"
-                                                 , 3000, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "packets"
+                            , NULL
+                            , "packets"
+                            , NULL
+                            , "IPv6 Packets"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3000
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -759,8 +1020,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_ip6_fragsout = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.fragsout");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "fragsout", NULL, "fragments", NULL, "IPv6 Fragments Sent"
-                                                 , "packets/s", 3010, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "fragsout"
+                            , NULL
+                            , "fragments"
+                            , NULL
+                            , "IPv6 Fragments Sent"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3010
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "ok", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -783,8 +1056,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_ip6_fragsin = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.fragsin");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "fragsin", NULL, "fragments", NULL, "IPv6 Fragments Reassembly"
-                                                 , "packets/s", 3011, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "fragsin"
+                            , NULL
+                            , "fragments"
+                            , NULL
+                            , "IPv6 Fragments Reassembly"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3011
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "ok", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -816,8 +1101,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_ip6_errors = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.errors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "errors", NULL, "errors", NULL, "IPv6 Errors", "packets/s"
-                                                 , 3002, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "errors"
+                            , NULL
+                            , "errors"
+                            , NULL
+                            , "IPv6 Errors"
+                            , "packets/s"
+                            , "macos"
+                            , "sysctl"
+                            , 3002
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
                     rrdset_flag_set(st, RRDSET_FLAG_DETAIL);
 
                     rrddim_add(st, "InDiscards", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -863,8 +1160,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6 = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmp");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmp", NULL, "icmp", NULL, "IPv6 ICMP Messages", "messages/s"
-                                                 , 10000, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmp"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 ICMP Messages"
+                            , "messages/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10000
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -882,8 +1191,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_redir = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmpredir");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmpredir", NULL, "icmp", NULL, "IPv6 ICMP Redirects"
-                                                 , "redirects/s", 10050, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmpredir"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 ICMP Redirects"
+                            , "redirects/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10050
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "received", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "sent", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -912,8 +1233,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_errors = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmperrors");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmperrors", NULL, "icmp", NULL, "IPv6 ICMP Errors"
-                                                 , "errors/s", 10100, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmperrors"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 ICMP Errors"
+                            , "errors/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10100
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InErrors", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutErrors", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -952,8 +1285,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_echos = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmpechos");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmpechos", NULL, "icmp", NULL, "IPv6 ICMP Echo", "messages/s"
-                                                 , 10200, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmpechos"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 ICMP Echo"
+                            , "messages/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10200
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InEchos", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutEchos", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -979,8 +1324,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_router = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmprouter");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmprouter", NULL, "icmp", NULL, "IPv6 Router Messages"
-                                                 , "messages/s", 10400, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmprouter"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 Router Messages"
+                            , "messages/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10400
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InSolicits", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutSolicits", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1006,8 +1363,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_neighbor = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmpneighbor");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmpneighbor", NULL, "icmp", NULL, "IPv6 Neighbor Messages"
-                                                 , "messages/s", 10500, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmpneighbor"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 Neighbor Messages"
+                            , "messages/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10500
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InSolicits", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "OutSolicits", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1039,8 +1408,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                 do_icmp6_types = CONFIG_BOOLEAN_YES;
                 st = rrdset_find_localhost("ipv6.icmptypes");
                 if (unlikely(!st)) {
-                    st = rrdset_create_localhost("ipv6", "icmptypes", NULL, "icmp", NULL, "IPv6 ICMP Types"
-                                                 , "messages/s", 10700, update_every, RRDSET_TYPE_LINE);
+                    st = rrdset_create_localhost(
+                            "ipv6"
+                            , "icmptypes"
+                            , NULL
+                            , "icmp"
+                            , NULL
+                            , "IPv6 ICMP Types"
+                            , "messages/s"
+                            , "macos"
+                            , "sysctl"
+                            , 10700
+                            , update_every
+                            , RRDSET_TYPE_LINE
+                    );
 
                     rrddim_add(st, "InType1", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
                     rrddim_add(st, "InType128", NULL, 1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1081,8 +1462,20 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             st = rrdset_find_localhost("system.uptime");
 
             if(unlikely(!st)) {
-                st = rrdset_create_localhost("system", "uptime", NULL, "uptime", NULL, "System Uptime", "seconds", 1000
-                                             , update_every, RRDSET_TYPE_LINE);
+                st = rrdset_create_localhost(
+                        "system"
+                        , "uptime"
+                        , NULL
+                        , "uptime"
+                        , NULL
+                        , "System Uptime"
+                        , "seconds"
+                        , "macos"
+                        , "sysctl"
+                        , 1000
+                        , update_every
+                        , RRDSET_TYPE_LINE
+                );
                 rrddim_add(st, "uptime", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
             else rrdset_next(st);
