@@ -1,6 +1,8 @@
 #ifndef NETDATA_PLUGIN_CHECKS_H
 #define NETDATA_PLUGIN_CHECKS_H 1
 
+#ifdef NETDATA_INTERNAL_CHECKS
 void *checks_main(void *ptr);
+#endif // NETDATA_INTERNAL_CHECKS
 
 #endif /* NETDATA_PLUGIN_PROC_H */
