@@ -274,6 +274,10 @@ static inline int is_machine_guid_blacklisted(const char *guid) {
     return 0;
 }
 
+char *registry_get_this_machine_hostname(void) {
+    return registry.hostname;
+}
+
 char *registry_get_this_machine_guid(void) {
     static char guid[GUID_LEN + 1] = "";
 
