@@ -22,25 +22,9 @@ void rrdcalctemplate_link_matching(RRDSET *st) {
     }
 }
 
-inline void rrdcalctemplate_free(RRDHOST *host, RRDCALCTEMPLATE *rt) {
+inline void rrdcalctemplate_free(RRDCALCTEMPLATE *rt) {
     if(unlikely(!rt)) return;
 
-    debug(D_HEALTH, "Health removing template '%s' of host '%s'", rt->name, host->hostname);
-
-    if(host->templates == rt) {
-        host->templates = rt->next;
-    }
-    else {
-        RRDCALCTEMPLATE *t;
-        for (t = host->templates; t && t->next != rt; t = t->next ) ;
-        if(t) {
-            t->next = rt->next;
-            rt->next = NULL;
-        }
-        else
-            error("Cannot find RRDCALCTEMPLATE '%s' linked in host '%s'", rt->name, host->hostname);
-    }
-
     expression_free(rt->calculation);
     expression_free(rt->warning);
     expression_free(rt->critical);
@@ -59,4 +43,26 @@ inline void rrdcalctemplate_free(RRDHOST *host, RRDCALCTEMPLATE *rt) {
     freez(rt);
 }
 
+inline void rrdcalctemplate_unlink_and_free(RRDHOST *host, RRDCALCTEMPLATE *rt) {
+    if(unlikely(!rt)) return;
+
+    debug(D_HEALTH, "Health removing template '%s' of host '%s'", rt->name, host->hostname);
+
+    if(host->templates == rt) {
+        host->templates = rt->next;
+    }
+    else {
+        RRDCALCTEMPLATE *t;
+        for (t = host->templates; t && t->next != rt; t = t->next ) ;
+        if(t) {
+            t->next = rt->next;
+            rt->next = NULL;
+        }
+        else
+            error("Cannot find RRDCALCTEMPLATE '%s' linked in host '%s'", rt->name, host->hostname);
+    }
+
+    rrdcalctemplate_free(rt);
+}
+
 
