@@ -4,7 +4,7 @@
 extern void signals_init(void);
 extern void signals_block(void);
 extern void signals_unblock(void);
-extern void signals_handle(void);
 extern void signals_reset(void);
+extern void signals_handle(void) NORETURN;
 
 #endif //NETDATA_SIGNALS_H
