@@ -15,8 +15,11 @@ typedef void SIMPLE_PATTERN;
 // should be considered PREFIX matches.
 extern SIMPLE_PATTERN *simple_pattern_create(const char *list, SIMPLE_PREFIX_MODE default_mode);
 
+// test if string str is matched from the pattern and fill 'wildcarded' with the parts matched by '*'
+extern int simple_pattern_matches_extract(SIMPLE_PATTERN *list, const char *str, char *wildcarded, size_t wildcarded_size);
+
 // test if string str is matched from the pattern
-extern int simple_pattern_matches(SIMPLE_PATTERN *list, const char *str);
+#define simple_pattern_matches(list, str) simple_pattern_matches_extract(list, str, NULL, 0)
 
 // free a simple_pattern that was created with simple_pattern_create()
 // list can be NULL, in which case, this does nothing.
